#!perl

# P. 103 Algorithmic Beauty of Plants
# ABOP (c)1990 Springer-Verlag

# Modeling a sunflower head - a more complex parametric L-system.
# Even though I only designate one file for all the petals, the extra
# characters were kept for consistency with the book. (The petal files
# themselves may or may not be included in the distribution with this
# Perl file. Nor may they even be of the correct type depending on the
# particular L-system viewer/interpreter implementation.)

# Be careful to use the logical AND operator "&&" rather than the
# bitwise operator "&" when copying from "formal" L-system specs.

# Also, this may take a while to process (huge recursion level).

# Perl L-System Implementation by:
# Sean O'Malley
# http://www.geocities.com/~ffrog

##################### DEFINES ########################

# Color table & starting color (first color index is 1).
# RGB-ordered triplets (0-255 scale).
$colorstart = 1;
@colortable = ( 133, 92, 46,
                0, 255, 0 );

# Output file
$file = 'turtle.lsd';

# Recursion level
$recurse = 615;

# Pre-defined shapes. Format: ( CHARACTER, PATH, SCALE )
@shapes = ( 'S', 'sunseed.dxf', '0.1',
            'R', 'sunflor.dxf', '0.1',
            'M', 'sunpetl.dxf', '0.1',
            'N', 'sunpetl.dxf', '0.1',
            'O', 'sunpetl.dxf', '0.1',
            'P', 'sunpetl.dxf', '0.1' );

# Constructor (initial string).
$constructor = "A(0)";

# Enumerate the functions used in the L-system here.
$functions = 'AC';

sub A {
   local($n) = $_[0];
   return "+(137.5)[f(".sqrt($n).")C($n)]A(".($n+1).")";
}

sub C {
   local($n) = $_[0];
   if ($n <= 440) {
      return "~S";
   } elsif ($n > 440 && $n <= 565) {
      return "~R";
   } elsif ($n > 565 && $n <= 580) {
      return "~M";
   } elsif ($n > 580 && $n <= 595) {
      return "~N";
   } elsif ($n > 595 && $n <= 610) {
      return "~O";
   } else {
      return "~P";
   }
}

######################################################

if (!-e "param.pl") {
   print "Can't find PARAM.PL:\n";
   die "It contains the necessary code to run this.\n";
}

open(COMMANDS, "param.pl");
undef $/;
eval <COMMANDS>;
$/ = "\n";
close(COMMANDS);
