#!perl

# P. 127 Algorithmic Beauty of Plants
# ABOP (c)1990 Springer-Verlag

# Rose leaf
# Illustrates variable parameterlist-length parametric L-systems.

# Perl L-System Implementation by:
# Sean O'Malley
# http://www.geocities.com/~ffrog

##################### DEFINES ########################

# Angle (outputted to turtle information file).
# Leave at 0 if this should not be outputted.
$angle = 60;

# Output file
$file = 'turtle.lsd';

# Recursion level
$recurse = 25;

# Constructor (initial string).
$constructor = "[{A(0,0).}][{A(0,1).}]";

# Defines. Even constants should have quotation marks.
$LA = "5";
$RA = "1.15";
$LB = "1.3";
$RB = "1.25";
$LC = "3";
$RC = "1.19";

# Enumerate the characters used by (function) rules in this variable.
$functions = "ABG";

# RULES

sub A {
   local($t, $d) = @_;
   if ($d == 0) {
      return ".G($LA,$RA).[+B($t)G($LC,$RC,$t).}][+B($t){.]A(".($t+1).",$d)";
   } else {
      return ".G($LA,$RA).[-B($t)G($LC,$RC,$t).}][-B($t){.]A(".($t+1).",$d)";
   }
}

sub B {
   local($t) = $_[0];
   if ($t > 0) {
      return "G($LB,$RB)B(".($t-1).")";
   } else {
      # undefined?
      return '';
   }
}

sub G {
   local($s, $r, $t) = @_;
   if (defined $t) {
      if ($t > 1) {
         return "G(".($s*$r).",$r,".($t-1).")";
      } else {
         # undefined?
         return '';
      }
   } else {
      return "G(".($s*$r).",$r)";
   }
}

######################################################

if (!-e "param.pl") {
   print "Can't find PARAM.PL:\n";
   die "It contains the necessary code to run this.\n";
}

open(COMMANDS, "param.pl");
undef $/;
eval <COMMANDS>;
$/ = "\n";
close(COMMANDS);
