#!perl

# P. 30 Algorithmic Beauty of Plants
# ABOP (c)1990 Springer-Verlag

# Shows the use of stochastic L-systems. Note that the main Perl
# code automatically seeds the random number generator, so it is
# unnecessary in each individual file. The rand(x) function returns
# a floating-point number between 0 and x. (If omitted, x defaults to 1.)

# This is a stochastic modification of the L-system on p. 27.
# See Rule C for how to implement stochastic...icity.

# Perl L-System Implementation by:
# Sean O'Malley
# http://www.geocities.com/~ffrog

##################### DEFINES ########################

# Color table & starting color (starting index is 1).
# RGB-ordered triplets (0-255 scale)
$colorstart = 3;
@colortable = ( 255, 255, 255,
                255, 255, 0,
                153, 102, 51,
                0, 255, 0 );

# Angle (outputted to turtle information file).
# Leave at 0 if this should not be outputted.
$angle = 18;

# Output file
$file = 'turtle.lsd';

# Recursion level
$recurse = 5;

# Constructor (initial string).
$constructor = "A";

# Enumerate the characters used by (function) rules in this variable.
$functions = "ABCDEHI";

# RULES
sub A {
   # Plant
   return "B+[A+E]--//[--D]B[++D]-[AE]++AE";
}

sub B {
   # Internode
   return "FC[//&&D][//^^D]FC";
}

sub C {
   # Segment (stochastic)
   local($prob) = rand(1);
   if ($prob < 0.33) {
      return "C[//&&D][//^^D]FC";
   } elsif ($prob < 0.66) {
      return "CFC";
   } else {
      return "C";
   }
}

sub D {
   # Leaf
   return "['{+f-ff-f+|+f-ff-f}]";
}

sub E {
   # Flower
   return "[&&&H`/I////I////I////I////I]";
}

sub H {
   # Pedicel
   return "FF";
}

sub I {
   # Wedge
   return "[`^F][{&&&&-f+f|-f+f}]";
}

######################################################

if (!-e "param.pl") {
   print "Can't find PARAM.PL:\n";
   die "It contains the necessary code to run this.\n";
}

open(COMMANDS, "param.pl");
undef $/;
eval <COMMANDS>;
$/ = "\n";
close(COMMANDS);
