#!perl

# Base code for parametric L-systems.
# Don't edit this file unless you know what you're doing. ;-)
# See included examples for how to set up parametric L-systems
# using this; normally this file will never need to be modified.

# Sean O'Malley
# June, 2000
# http://www.geocities.com/~ffrog

######################### CODE ##############################

$| = 1;
if ($] < 5) { die "This requires Perl 5+.\n"; }

Main();

sub Main {
    local($final, $lp, $index, $tstring);

    # Seed the random number generator for stochastic L-systems.
    srand;

#    if (-e $file) {
#       print "Not overwriting \"$file\". Please remove file.\n";
#       exit;
#    }

    print "Creating string ... ";
    $final = Iter($constructor, 0, $recurse);
    print "Finished.\n";
    print "Final string is ".(length($final))." characters long.\n";
    print "Outputting file ... ";

    open(OUTFILE, ">$file");
    if ($angle != 0) { print OUTFILE "ANGLE\n$angle\n\n"; }

    if (defined @colortable) {
       if (!defined $colorstart) { $colorstart = 1; }
       $colorstart--;
       print OUTFILE "COLORSTART\n$colorstart\n\n";
       print OUTFILE "COLORTABLESIZE\n".(($#colortable + 1) / 3)."\n\n";
       $index = 0;
       for ($lp = 0; $lp < ($#colortable - 1); $lp += 3) {
          print OUTFILE "COLORTABLE$index\n$colortable[$lp] $colortable[$lp+1] $colortable[$lp+2]\n\n";
          $index++;
       }
    }

    if (defined @shapes) {
       $index = 0;
       $tstring = '';
       for ($lp = 0; $lp < ($#shapes - 1); $lp += 3) {
          $tstring .= $shapes[$lp];
          print OUTFILE "SHAPE$shapes[$lp]\n$shapes[$lp+1]\n\n";
          print OUTFILE "SHAPESCALE$shapes[$lp]\n$shapes[$lp+2]\n\n";
       }
       print OUTFILE "SHAPEENUM\n$tstring\n\n";
    }

    if (defined $tropism_vector) {
       print OUTFILE "TROPISM_VECTOR\n$tropism_vector\n\n";
       if (defined $tropism_e) { print OUTFILE "TROPISM_E\n$tropism_e\n\n"; }
    }

    print OUTFILE "CONSTRUCTOR_LENGTH\n".(length($final))."\n\n";

    if (defined replacer) {
       while (($what, $with) = each %replacer) {
          $final =~ s/$what/$with/g;
       }
    }

    print OUTFILE "DATA\n$final\n\n";
    close(OUTFILE);
    print "Finished.\n";
}

sub Iter {
   local($string, $nest, $max) = @_;
   local($newstring, $lp, $com, $char, $param, $len);
   local(@params);

   # print "Recursion: $nest\n";

   if ($nest == $max) { return $string; }

   $newstring = '';
   $len = length($string);

   for ($lp = 0; $lp < $len; $lp++) {
      $com = substr($string, $lp, 1);

      if (substr($string, $lp+1, 1) eq '(') {
         $lp += 2;
         $param = '';
         $char = substr($string, $lp, 1);
         while ($char ne ')') {
            $param .= $char;
            $lp++;
            $char = substr($string, $lp, 1);
         }
         @params = SplitParams($param);
      } else {
         $param = '';
      }

      if ($com =~ /[$functions]/) {
         if ($param ne '') {
            $newstring .= eval($com."(".EvalCommaSeparate(@params).")");
         } else {
            $newstring .= eval("$com()");
         }
      } else {
         if ($param eq '') {
            $newstring .= $com;
         } else {
            $newstring .= $com."(".EvalCommaSeparate(@params).")";
         }
      }
   }

   return Iter($newstring, $nest + 1, $max);
}

sub EvalCommaSeparate {
   # Evals each section of the passed array, then returns a string
   # containing the evaluated values with commas in between.
   local(@array) = @_;
   local($lp, $max, $string);

   $max = $#array;
   $string = '';
   for ($lp = 0; $lp <= $max; $lp++) {
      $string .= eval($array[$lp]);
      if ($lp < $max) { $string .= ","; }
   }
   return $string;
}

sub SplitParams {
   # Probably a much simpler way to do this with Perl. ;-)
   # Call with a single string (e.g., "bla, squid, 1.0") and it
   # returns an array of the things between the commas.
   local($string) = $_[0];
   local(@temp, $lp, $char, $len, $index);

   $len = length($string);
   $index = 0;
   $temp[0] = '';
   for ($lp = 0; $lp < $len; $lp++) {
      $char = substr($string, $lp, 1);
      if ($char eq ',') {
         $index++;
         $temp[$index] = '';
      } else {
         $temp[$index] .= $char;
      }
   }

   return @temp;
}

sub QuoteDangerous {
   $_ = $_[0];
   s/(\W)/\\$1/g;
   return $_;
}
