#!perl

# This template contains almost all the features of the PARAM.PL
# L-system interpreter: defined constants, exporting tropism information,
# functions, how to deal with illegal (to Perl) function characters, etc.
# Except where noted, any unnecessary features can simply be erased (i.e.,
# the tropism variables, definitions, the %replacer array, etc).

# Important features not shown here:

# Stochastic L-systems: See P28EXAM.PL and P30EXAM.PL
# Variable parameterlist-length rules: See P126EXAM.PL
# Exporting a color table: See P26EXAM.PL
# Pre-defined DXF objects: See P103EXAM.PL

# This template is a copy of P60EXAM.PL. All original parameters have
# been left in place to show where everything goes.

# Perl L-System Implementation by:
# Sean O'Malley
# http://www.geocities.com/~ffrog

##################### DEFINES ########################

# Angle (outputted to turtle information file).
# Leave at 0 if this should not be outputted.
$angle = 0;

# Output file
$file = 'turtle.lsd';

# Recursion level
$recurse = 6;

# Constructor (initial string).
$constructor = "Q(1)F(200)/(45)A";

# Defines. Even constants should have quotation marks.
$d1 = "112.5";
$d2 = "157.5.0";
$a = "22.5";
$lr = "1.79";
$vr = "1.732";

# Tropism
$tropism_vector = "-0.02 -1.0 0.0";
$tropism_e = "0.27";

# Enumerate the characters used by (function) rules in this variable.
$functions = "AFQ";

# Replace all Q's (dummy character) in the final string with with !'s since
# ! is not allowed as a function name. This associative array can be
# enlarged for more substitutions. May be necessary to quote some dangerous
# characters.
%replacer = ( "Q", "!" );

# RULES

sub A {
   return "Q($vr)F(50)[&($a)F(50)A]/($d1)[&($a)F(50)A]/($d2)[&($a)F(50)A]";
}

sub F {
   local($l) = $_[0];
   return "F(".($l*$lr).")";
}

sub Q {
   # AKA "!"
   local($w) = $_[0];
   return "Q(".($w*$vr).")";
}

######################################################

if (!-e "param.pl") {
   print "Can't find PARAM.PL:\n";
   die "It contains the necessary code to run this.\n";
}

open(COMMANDS, "param.pl");
undef $/;
eval <COMMANDS>;
$/ = "\n";
close(COMMANDS);
